#!/usr/bin/env python

import PIL.Image
import sys

def get_image(input_filename, width, height):

    i = PIL.Image.open(input_filename)
    if i.size != (width, height):
        i.thumbnail((width, height), PIL.Image.ANTIALIAS)

    # Convert to monochrome.

    i = i.convert("1")

    all_values = []
    values = []
    last = None
    x = 0
    byte = 0

    for value in i.getdata():

        byte = (byte << 1) + (value == 255 and 1 or 0)
        x += 1

        if x % 8 == 0:
            values.append(byte)
            byte = 0

        if x == width:
            x = 0
            byte = 0
            all_values.append(values)
            values = []

    return all_values

if __name__ == "__main__":
    import sys, os

    if len(sys.argv) < 3:
        print sys.argv[0], "<output basename> <image filename>..."
        sys.exit(1)

    output_basename = sys.argv[1]
    input_filenames = sys.argv[2:]
    width = 264
    height = 176
    number_of_images = len(input_filenames)

    f = open("%s.c" % output_basename, "wb")

    try:
        # Write the image data.

        f.write("#include <stdint.h>\n")
        f.write("const uint8_t image_data[%d][%d][%d] = {\n" % (number_of_images, height, width / 8))
        for input_filename in input_filenames:
            f.write("{\n")
            data = get_image(input_filename, width, height)
            for y in range(0, height):
                values = data[y]
                f.write("{\n")
                f.write(", ".join([("0x%02x" % v) for v in values]))
                f.write("},\n")
            f.write("},\n")
        f.write("};\n")

    finally:
        f.close()

    f = open("%s.h" % output_basename, "wb")

    try:
        f.write("#include <stdint.h>\n")
        f.write("extern const uint8_t image_data[%d][%d][%d];\n" % (number_of_images, height, width / 8))
        f.write("#define MAX_IMAGE %d\n" % (number_of_images - 1))
        f.write("#define NUMBER_OF_IMAGES %d\n" % number_of_images)

    finally:
        f.close()

# vim: tabstop=4 expandtab shiftwidth=4
